﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Reflection;

using Prova3Lib;

namespace Prova3
{
    public class DrawerFactory
    {
        private readonly Dictionary<Type, IDrawer> _drawersDictionary = new Dictionary<Type, IDrawer>();
        private readonly static DrawerFactory _instance = new DrawerFactory();

        private DrawerFactory()
        {
            foreach (Type shapeType in ShapeFactory.Instance.GetShapesTypes())
                _drawersDictionary.Add(shapeType, null);

            foreach (Assembly assembly in AppDomain.CurrentDomain.GetAssemblies())
            {
                foreach (Type type in assembly.GetTypes())
                {
                    if (type.GetInterface("Prova3Lib.IDrawer") == null)
                        continue;
                    if (type.IsAbstract)
                        continue;
                    object[] attributes = type.GetCustomAttributes(typeof(DrawerAttribute), false);
                    if (attributes.Length == 0)
                        continue;
                    DrawerAttribute drawerAttribute = (DrawerAttribute) attributes[0];
                    //Se ci sono diverse shape con lo stesso nome in diversi namespace...
                    if (!_drawersDictionary.ContainsKey(drawerAttribute.ShapeType))
                        throw new ApplicationException("!_drawerDictionary.ContainsKey(drawerAttribute.ShapeType)");
                    if (_drawersDictionary[drawerAttribute.ShapeType] != null)
                        throw new ApplicationException("_drawerDictionary[drawerAttribute.ShapeType] != null");
                    IDrawer drawer = (IDrawer) Activator.CreateInstance(type);
                    _drawersDictionary[drawerAttribute.ShapeType] = drawer;
                }
            }

            foreach (Type shapeType in ShapeFactory.Instance.GetShapesTypes())
                if (_drawersDictionary[shapeType] == null)
                    throw new ApplicationException("_drawerDictionary[shapeType] == null");
        }

        public static DrawerFactory Instance
        {
            get { return _instance; }
        }

        public IDrawer GetDrawerFor(IShape shape)
        {
            if (!_drawersDictionary.ContainsKey(shape.GetType()))
                throw new ApplicationException("!_drawersDictionary.ContainsKey(shape.GetType())");
            return _drawersDictionary[shape.GetType()];
        }
    }
}
