﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using Prova3Lib;
using Prova3Lib.MVC;

namespace Prova3
{
    public class Model : ModelBase
    {
        private readonly List<IShape> _shapes = new List<IShape>();

        public Model()
        {
            foreach (Type shapeType in ShapeFactory.Instance.GetShapesTypes())
                Add((IShape) Activator.CreateInstance(shapeType));
            //foreach (string shapeName in ShapeFactory.Instance.GetShapesNames())
            //    Add(ShapeFactory.Instance.CreateShape(shapeName));
        }

        public IEnumerable<IShape> GetShapes()
        {
            return _shapes;
        }

        public void Add(IShape shape)
        {
            if (shape == null)
                throw new ArgumentNullException("shape");
            if (_shapes.Contains(shape))
                throw new ApplicationException("_shapes.Contains(shape)");
            shape.Changed += shape_Changed;
            _shapes.Add(shape);
            OnChanged();
        }

        public void Remove(IShape shape)
        {
            if (shape == null)
                throw new ArgumentNullException("shape");
            if (!_shapes.Contains(shape))
                throw new ApplicationException("!_shapes.Contains(shape)");
            shape.Changed -= shape_Changed;
            _shapes.Remove(shape);
            OnChanged();
        }

        private void shape_Changed(object sender, EventArgs e)
        {
            OnChanged();
        }
    }
}
