﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Reflection;

using Prova3Lib;

namespace Prova3
{
    public class ShapeFactory
    {
        private readonly Dictionary<string, Type> _shapesDictionary = new Dictionary<string, Type>();
        private readonly static ShapeFactory _instance = new ShapeFactory();

        private ShapeFactory()
        {
            foreach (Assembly assembly in AppDomain.CurrentDomain.GetAssemblies())
            {
                foreach (Type type in assembly.GetTypes())
                {
                    if (type.GetInterface("Prova3Lib.IShape") == null)
                        continue;
                    if (type.IsAbstract)
                        continue;
                    //Se ci sono diverse shape con lo stesso nome in diversi namespace...
                    _shapesDictionary.Add(type.Name, type);
                }
            }
        }

        public static ShapeFactory Instance
        {
            get { return _instance; }
        }

        public IEnumerable<string> GetShapesNames()
        {
            return _shapesDictionary.Keys;
        }

        public IEnumerable<Type> GetShapesTypes()
        {
            return _shapesDictionary.Values;
        }

        public IShape CreateShape(string typeName)
        {
            if (!_shapesDictionary.ContainsKey(typeName))
                throw new ApplicationException("!_shapesDictionary.ContainsKey(typeName)");
            Type shapeType = _shapesDictionary[typeName];
            return (IShape) Activator.CreateInstance(shapeType);
        }
    }
}
